import React, {Component} from 'react';

import {
    Button,
    FlatList,
    Image,
    SafeAreaView,
    ScrollView,
    StyleSheet,
    Text,
    TextInput,
    ToastAndroid,
    TouchableNativeFeedback,
    View
} from 'react-native';

import myket from 'iab-myket-reactnative';

const styles = StyleSheet.create({
    header: {
        height: 120,
        paddingTop: 22,
        paddingStart: 22,
        flexDirection: 'row',
        alignItems: 'center'
    },
    item: {
        flexDirection: 'row-reverse',
        marginTop: 1,
        padding: 8,
        backgroundColor: "gray",
        height: 80
    },
    text: {
        textAlign: "right"
    }
});

/**
 retrieve RSA public key from Myket panel, visit https://myket.ir/kb/pages/get-pki-public-key/ for more information
 */
const base64RSA = "MIGeMA0GCSqGSIb3DQEBAQUAA4GMADCBiAKBgOW5KR56WBWCb5K+yyVDnh/7op0FY4zmM93CWz3xFhgUJe2WXM/8MgpTHiDxrj2Mkgt9bg30qZDtT8gzDHiTgNv6G7pZBDWuyKEariGbbQgoCoeaq3GBcNsQf418jsvOfPjzZ7Rpcl/+9ZPsp1kbJVOmZxnwAZx/wnkUduwfuf8hAgMBAAE=";


const itemList = [
    {sku: "gas", icon: require('./assets/buy_gas.png'), consumable: true},
    {sku: "premium", icon: require('./assets/upgrade_app.png'), consumable: false},
    {sku: "infinite_gas", icon: require('./assets/get_infinite_gas.png'), consumable: false}
];

const items = new Map();
itemList.map((x) => items[x.sku] = x);

class App extends Component {
    didMount = false;
    state = {
        dynamicPriceToken: "",
        log: "",
        waiting: true,
        vehicle: {gas: 4, gasIcon: require("./assets/gas4.png"), skin: require("./assets/free.png")}
    };

    componentDidMount() {
        console.log('mount')
        this.didMount = true;
        myket
            .connect(base64RSA)
            .catch((error) => {
                /**
                 * myket is not installed or RSA public key is null or empty
                 */
                this.log(error)
            }).then(this.retrieverProducts);
    }

    componentWillUnmount() {
        myket.disconnect().catch(() => this.log);
    }

    retrieverProducts = async () => {

        let productIds = itemList.map(x => x.sku);

        let inventory = await myket.queryPurchaseProduct(true, productIds).catch((error) => {
            this.log(error.toString())
            this.setState({
                waiting: false
            });

        });

    

        inventory.allProducts.map(s => {
            items[s.productId].title = s.title;
            items[s.productId].price = s.price;
            items[s.productId].description = s.description;
        });

        /**
         * It is recommended to consume products that User have already bought, immediately after querying the inventory
         */
        inventory.allPurchases.map((purchase) => {
            this.consumePurchase(purchase)
        })


        this.setState({
            waiting: false
        });
    }

    purchase = async (purchase) => {
    
        if (purchase.sku === "gas") {
            if (this.state.vehicle.gas >= 4) {
                this.log(`Fuel tank is full !`)
                return;
            }
        }

        /*
         TODO: for security, generate your payload here for verification.Since this is a SAMPLE, we just use
          * an empty string, but on a production app you should carefully generate this.
         */
         myket.purchaseProduct(purchase.sku, "").then((message) => {
            console.log("called consumed!"+message)
            this.consumePurchase(message);
        }).catch((error)=>{
            this.setState({
                waiting: false
            });
            this.log(error.message)
        });

    }


    consumePurchase(purchase) {
        this.setState({
            waiting: true
        });
        myket.consumePurchase(purchase).then((model) => {
            this.changeAssetValue(model)
            ToastAndroid.show('consume successful', ToastAndroid.SHORT);
            this.setState({
                waiting: false
            });
        }).catch((error) => {
            this.log(error.message)
            this.setState({
                waiting: false
            });
        });
    }

    changeAssetValue(purchase) {
        console.log("is "+purchase.sku)

        if (purchase.sku === "gas") {

            this.seGas(this.state.vehicle.gas + 1);
        } else if (purchase.sku === "infinite_gas") {
            this.seGas(5);
        } else if (purchase.sku === "premium") {
            this.setState({
                vehicle: Object.assign(this.state.vehicle, {skin: require("./assets/premium.png")})
            });
        }
    }

    seGas = (gas) => {
        if (gas < 0) {
            this.log(`Fuel tank is empty !`);
            return;
        }

        let icon = require("./assets/gas0.png");
        switch (gas) {
            case 1:
                icon = require("./assets/gas1.png");
                break;
            case 2:
                icon = require("./assets/gas2.png");
                break;
            case 3:
                icon = require("./assets/gas3.png");
                break;
            case 4:
                icon = require("./assets/gas4.png");
                break;
            case 5:
                icon = require("./assets/gas_inf.png");
                break;
        }
        this.setState({
            vehicle: Object.assign(this.state.vehicle, {gas: gas, gasIcon: icon}),
        });
    }

    drive = () => {
        if (this.state.vehicle.gas < 5) {
            this.seGas(this.state.vehicle.gas - 1)
        }
    }

    log = (text) => {
        let txt = "";
        if (typeof text === 'object') {
            txt += JSON.stringify(text);
        } else {
            txt = text;
        }

        console.log(text);
        if (this.didMount) {
            this.setState({
                log: this.state.log + "\n" + txt
            });
        }
    }

    render() {
        return (
            <SafeAreaView style={{flex: 1, backgroundColor: "dimgray"}}>
                <View style={{flexDirection: 'column', flex: 1}}>
                    <View style={styles.header}>
                        <View style={{flexDirection: "column", alignItems: 'center'}}>
                            <Image resizeMode={'contain'} style={{width: "100%", height: 56}}
                                   source={this.state.vehicle.skin}/>
                            <Image resizeMode={'contain'} style={{width: 128}} source={this.state.vehicle.gasIcon}/>
                        </View>
                        <View style={{flex: 1, alignItems: 'center', paddingBottom: 22}}>
                            <Button title={"یه دوری بزن!"} onPress={() => this.drive()}/>
                        </View>
                    </View>
                    <View>
                        <FlatList
                            data={itemList}
                            renderItem={({item}) => {
                                return <TouchableNativeFeedback onPress={() => this.purchase(item)}>
                                    <View style={styles.item}>
                                        <View style={{flexDirection: 'column', flex: 0.8}}>
                                            <Text style={styles.text}>{item.title}</Text>
                                            {item.id == "dynamic_price"
                                                ? <TextInput style={{borderWidth: 1, height: 36}} value={this.state.log}
                                                             onChangeText={(t) => {
                                                                 this.setState({dynamicPriceToken: t});
                                                             }}/>
                                                : <Text style={styles.text}>{item.description}</Text>}
                                        </View>
                                        <View style={{flexDirection: 'column', flex: 0.3, alignItems: 'center'}}>
                                            <Image resizeMode={'contain'} style={{flex: 1}} source={item.icon}/>
                                            <View style={{height: 4}}/>
                                            <Text style={styles.text}>{item.price}</Text>
                                        </View>
                                    </View>
                                </TouchableNativeFeedback>
                            }}
                        />
                    </View>
                    <ScrollView style={{margin: 8}}>
                        <Text>{this.state.log}</Text>
                    </ScrollView>
                    <View style={{alignSelf: 'flex-end', right: 8, bottom: 8}}>
                        <Button title='Clear' onPress={() => this.setState({log: ""})}/>
                    </View>
                </View>
                {
                    this.state.waiting ? (
                        <View style={{
                            backgroundColor: 0xe2e2e2b2,
                            position: 'absolute',
                            right: 0,
                            left: 0,
                            top: 0,
                            bottom: 0,
                            justifyContent: 'center'
                        }}>
                            <Text style={{alignSelf: 'center', color: 'black'}}>Please wait ...</Text>
                        </View>) : null
                }
            </SafeAreaView>);
    }
}

export default App;